@@ -1,7 +1,7 @@
 /*  This file is part of the KDE project.
 
     Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-    Copyright (C) 2011 Trever Fischer <tdfischer@fedoraproject.org>
+    Copyright (C) 2011 Torrie Fischer <tdfischer@kde.org>
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -33,9 +33,7 @@
 #include <QtCore/QWaitCondition>
 #include <QtCore/QMutex>
 
-#include "phonon-config-gstreamer.h"
-
-QT_BEGIN_NAMESPACE
+#include "phonon-config-gstreamer.h" // krazy:exclude=includes
 
 class QTimer;
 typedef QMultiMap<QString, QString> TagMap;
@@ -110,32 +108,32 @@ public:
     bool hasInterface(Interface) const;
     QVariant interfaceCall(Interface, int, const QList<QVariant> &);
 #endif
-    bool isLoading()
+    bool isLoading() const
     {
         return m_loading;
     }
 
-    bool audioAvailable()
+    bool audioAvailable() const
     {
         return m_pipeline->audioIsAvailable();
     }
 
-    bool videoAvailable()
+    bool videoAvailable() const
     {
         return m_pipeline->videoIsAvailable();
     }
 
-    GstElement *audioGraph()
+    GstElement *audioGraph() const
     {
         return m_pipeline->audioGraph();
     }
 
-    GstElement *videoGraph()
+    GstElement *videoGraph() const
     {
         return m_pipeline->videoGraph();
     }
 
-    Pipeline *pipeline()
+    Pipeline *pipeline() const
     {
         return m_pipeline;
     }
@@ -181,12 +179,12 @@ protected:
     void loadingComplete();
     Q_INVOKABLE void setError(const QString &errorString, Phonon::ErrorType error = NormalError);
 
-    GstElement *audioElement()
+    GstElement *audioElement() const
     {
         return m_pipeline->audioPipe();
     }
 
-    GstElement *videoElement()
+    GstElement *videoElement() const
     {
         return m_pipeline->videoPipe();
     }
@@ -194,13 +192,13 @@ protected:
 private Q_SLOTS:
     void handleTrackCountChange(int tracks);
     void getSubtitleInfo(int stream);
+    void getAudioChannelInfo(int stream);
     void emitTick();
     void beginPlay();
     void autoDetectSubtitle();
+    void logWarning(const QString &);
 
     void handleEndOfStream();
-    void logWarning(const QString &);
-    void handleBuffering(int);
     void handleStateChange(GstState oldState, GstState newState);
     void handleDurationChange(qint64);
 
@@ -223,6 +221,10 @@ private:
     void changeTitle(const QString &format, int title);
     void changeSubUri(const Mrl &mrl);
 
+    QList<AudioChannelDescription> _iface_availableAudioChannels() const;
+    AudioChannelDescription _iface_currentAudioChannel() const;
+    void _iface_setCurrentAudioChannel(const AudioChannelDescription &channel);
+
     bool m_resumeState;
     State m_oldState;
     quint64 m_oldPos;
@@ -253,9 +255,10 @@ private:
     int m_availableTitles;
     int m_currentTitle;
     SubtitleDescription m_currentSubtitle;
+    AudioChannelDescription m_currentAudioChannel;
     int m_pendingTitle;
 
-    // When we emit aboutToFinish(), libphonon calls setNextSource. To achive gapless playback,
+    // When we emit aboutToFinish(), libphonon calls setNextSource. To achieve gapless playback,
     // the pipeline is immediately told to start using that new source. This can break seeking
     // since the aboutToFinish signal tends to be emitted around 15 seconds or so prior to actually
     // ending the current track.
@@ -290,6 +293,4 @@ private:
 }
 } //namespace Phonon::Gstreamer
 
-QT_END_NAMESPACE
-
 #endif // Phonon_GSTREAMER_MEDIAOBJECT_H
