@@ -29,8 +29,7 @@
 #include <QtGui/QPainter>
 #include <X11/Xlib.h>
 #include <gst/gst.h>
-#include <gst/interfaces/xoverlay.h>
-#include <gst/interfaces/propertyprobe.h>
+#include <gst/video/videooverlay.h>
 
 namespace Phonon
 {
@@ -40,10 +39,13 @@ namespace Gstreamer
 class OverlayWidget : public QWidget
 {
 public:
-    OverlayWidget(VideoWidget *videoWidget, X11Renderer *renderer) :
-                  QWidget(videoWidget),
-                  m_videoWidget(videoWidget),
-                  m_renderer(renderer) { }
+    OverlayWidget(VideoWidget *videoWidget, X11Renderer *renderer)
+            : QWidget(videoWidget)
+            , m_videoWidget(videoWidget)
+            , m_renderer(renderer)
+    {
+    }
+
     void paintEvent(QPaintEvent *) {
         Phonon::State state = m_videoWidget->root() ? m_videoWidget->root()->state() : Phonon::LoadingState;
         if (state == Phonon::PlayingState || state == Phonon::PausedState) {
@@ -53,6 +55,7 @@ public:
             painter.fillRect(m_videoWidget->rect(), m_videoWidget->palette().background());
         }
     }
+
 private:
     VideoWidget *m_videoWidget;
     X11Renderer *m_renderer;
@@ -65,10 +68,13 @@ X11Renderer::X11Renderer(VideoWidget *videoWidget)
     debug() << "Creating X11 overlay renderer";
     QPalette palette;
     palette.setColor(QPalette::Background, Qt::black);
-    m_videoWidget->setPalette(palette);
-    m_videoWidget->setAutoFillBackground(true);
+    videoWidget->setPalette(palette);
+    videoWidget->setAutoFillBackground(true);
     m_renderWidget->setMouseTracking(true);
-    m_videoSink = createVideoSink();
+    GstElement *videoSink = createVideoSink();
+    if (videoSink) {
+        setVideoSink(videoSink);
+    }
     aspectRatioChanged(videoWidget->aspectRatio());
     setOverlay();
 }
@@ -82,7 +88,7 @@ X11Renderer::~X11Renderer()
 
 GstElement* X11Renderer::createVideoSink()
 {
-    GstElement *videoSink = gst_element_factory_make ("xvimagesink", NULL);
+    GstElement *videoSink = gst_element_factory_make("xvimagesink", NULL);
     if (videoSink) {
         // Check if the xv sink is usable
         if (gst_element_set_state(videoSink, GST_STATE_READY) != GST_STATE_CHANGE_SUCCESS) {
@@ -100,29 +106,26 @@ GstElement* X11Renderer::createVideoSink()
     }
     QByteArray tegraEnv = qgetenv("TEGRA_GST_OPENMAX");
     if (!tegraEnv.isEmpty()) {
-        videoSink = gst_element_factory_make ("nv_gl_videosink", NULL);
+        videoSink = gst_element_factory_make("nv_gl_videosink", NULL);
     }
     if (!videoSink) {
-        videoSink = gst_element_factory_make ("ximagesink", NULL);
+        videoSink = gst_element_factory_make("ximagesink", NULL);
     }
 
-    gst_object_ref (GST_OBJECT (videoSink)); //Take ownership
-    gst_object_sink (GST_OBJECT (videoSink));
-
     return videoSink;
 }
 
 void X11Renderer::aspectRatioChanged(Phonon::VideoWidget::AspectRatio)
 {
     if (m_renderWidget) {
-        m_renderWidget->setGeometry(m_videoWidget->calculateDrawFrameRect());
+        m_renderWidget->setGeometry(videoWidget()->calculateDrawFrameRect());
     }
 }
 
 void X11Renderer::scaleModeChanged(Phonon::VideoWidget::ScaleMode)
 {
     if (m_renderWidget) {
-        m_renderWidget->setGeometry(m_videoWidget->calculateDrawFrameRect());
+        m_renderWidget->setGeometry(videoWidget()->calculateDrawFrameRect());
     }
 }
 
@@ -131,7 +134,7 @@ void X11Renderer::movieSizeChanged(const QSize &movieSize)
     Q_UNUSED(movieSize);
 
     if (m_renderWidget) {
-        m_renderWidget->setGeometry(m_videoWidget->calculateDrawFrameRect());
+        m_renderWidget->setGeometry(videoWidget()->calculateDrawFrameRect());
     }
 }
 
@@ -146,7 +149,7 @@ bool X11Renderer::eventFilter(QEvent *e)
     } else if (e->type() == QEvent::Resize) {
         // This is a workaround for missing background repaints
         // when reducing window size
-        m_renderWidget->setGeometry(m_videoWidget->calculateDrawFrameRect());
+        m_renderWidget->setGeometry(videoWidget()->calculateDrawFrameRect());
         windowExposed();
     }
     return false;
@@ -154,19 +157,15 @@ bool X11Renderer::eventFilter(QEvent *e)
 
 void X11Renderer::handlePaint(QPaintEvent *)
 {
-    QPainter painter(m_videoWidget);
-    painter.fillRect(m_videoWidget->rect(), m_videoWidget->palette().background());
+    QPainter painter(videoWidget());
+    painter.fillRect(videoWidget()->rect(), videoWidget()->palette().background());
 }
 
 void X11Renderer::setOverlay()
 {
-    if (m_videoSink && GST_IS_X_OVERLAY(m_videoSink)) {
+    if (videoSink() && GST_IS_VIDEO_OVERLAY(videoSink())) {
         WId windowId = m_renderWidget->winId();
-#if GST_VERSION >= GST_VERSION_CHECK(0,10,31,0)
-        gst_x_overlay_set_window_handle(GST_X_OVERLAY(m_videoSink), windowId);
-#else
-        gst_x_overlay_set_xwindow_id(GST_X_OVERLAY(m_videoSink), windowId);
-#endif // GST_VERSION
+        gst_video_overlay_set_window_handle(GST_VIDEO_OVERLAY(videoSink()), windowId);
     }
     windowExposed();
     m_overlaySet = true;
@@ -176,14 +175,16 @@ void X11Renderer::windowExposed()
 {
     // This can be invoked within a callchain in an arbitrary thread, so make
     // sure we call syncX() from the main thread
-    QMetaObject::invokeMethod(m_videoWidget, "syncX",
+    QMetaObject::invokeMethod(videoWidget(), "syncX",
                               Qt::QueuedConnection);
 
-    if (m_videoSink && GST_IS_X_OVERLAY(m_videoSink))
-        gst_x_overlay_expose(GST_X_OVERLAY(m_videoSink));
+    if (videoSink() && GST_IS_VIDEO_OVERLAY(videoSink())) {
+        gst_video_overlay_expose(GST_VIDEO_OVERLAY(videoSink()));
+    }
 }
 
 }
+
 } //namespace Phonon::Gstreamer
 
 #endif // Q_WS_QWS
